#!/bin/sh

[ -f /etc/profile ] && . /etc/profile >/dev/null

LAN_IF_NAME=""
LAN_IF_ADDR=""

ROUTER=""
WORK_DIR="/tmp/xunyou"
BASEDIR="$(cd "$(dirname "$0")"; pwd -P)"
XUNYOU_CHAIN="XUNYOU"
XUNYOU_TCP_DNS_CHAIN="XUNYOU_TCP_DNS"
XUNYOU_ACC_CHAIN="XUNYOU_ACC"
XUNYOU_PROXY_CHAIN="XUNYOU_PROXY"
XUNYOU_CDN_CHAIN="XUNYOU_CDN"
NFT_XUNYOU_TABLE="xunyou_table"
NFT_XUNYOU_TABLE6="xunyou_table6"
NFT_XUNYOU_DNAT_CHAIN="xunyou_dnat_chain"
NFT_XUNYOU_DROP_CHAIN="xunyou_drop_chain"
NFT_XUNYOU_UDP_CHAIN="xunyou_udp_chain"
NFT_XUNYOU_CDN_CHAIN="xunyou_cdn_chain"
NFT_XUNYOU_SRCNAT_CHAIN="xunyou_srcnat_chain"
NFT_XUNYOU_INPUT_CHAIN="xunyou_input_chain"
NFT_XUNYOU_FORWARD_CHAIN="xunyou_forward_chain"

ACC_CONF="${WORK_DIR}/conf/accelerate"
XUNYOU_CONFIG="${BASEDIR}/xunyou.conf"
TMP_XUNYOU_CONFIG="/tmp/xunyou/conf/xunyou.conf"

ASUSWRT_MERLIN="merlin"
XIAOMI="xiaomi"
NETGEAR="netgear"
KOOLSHARE="koolshare"
ORAY="oray"
OPENWRT="openwrt"
NFT_CMD=""
USE_NFT="false"

get_json_value()
{
    local json=${1}
    local key=${2}
    local line=`echo ${json} | tr -d "\n " | awk -F"[][}{,]" '{for(i=1;i<=NF;i++) {if($i~/^"'${key}'":/) print $i}}' | tr -d '"' | sed -n 1p`
    local value=${line#*:}
    echo ${value}
}

env_init()
{   
    if [ -f "${XUNYOU_CONFIG}" ]; then
        local conf_router=$(grep "router=" "${XUNYOU_CONFIG}" | head -n1 | cut -d= -f2)
        local conf_model=$(grep "model=" "${XUNYOU_CONFIG}" | head -n1 | cut -d= -f2)
    else
        local conf_router=$(grep "router=" "${TMP_XUNYOU_CONFIG}" | head -n1 | cut -d= -f2)
        local conf_model=$(grep "model=" "${TMP_XUNYOU_CONFIG}" | head -n1 | cut -d= -f2)
    fi
    
    ROUTER="${conf_router}"
    case ${ROUTER} in
        ${ASUSWRT_MERLIN})
            LAN_IF_NAME="br0"
            ;;
        ${XIAOMI})
            LAN_IF_NAME="br-lan"
            ;;
        ${NETGEAR})
            LAN_IF_NAME="br0"
            ;;
        ${KOOLSHARE})
            LAN_IF_NAME="br-lan"
            ;;
        ${ORAY})
            LAN_IF_NAME="br-lan"
            ;;
        ${OPENWRT})
            LAN_IF_NAME="br-lan"
            ;;
        *)
            return 1
            ;;
    esac
    
    if [ -f "${BASEDIR}/lan.config" ]; then
        local lan_if_name=$(grep "lan_if_name=" "${BASEDIR}/lan.config" 2>/dev/null | head -n1 | cut -d= -f2)
        [ -n "${lan_if_name}" ] && LAN_IF_NAME="${lan_if_name}"
    elif [ -f "${WORK_DIR}/conf/lan.config" ]; then
        local lan_if_name=$(grep "lan_if_name=" "${WORK_DIR}/conf/lan.config" 2>/dev/null | head -n1 | cut -d= -f2)
        [ -n "${lan_if_name}" ] && LAN_IF_NAME="${lan_if_name}"
    fi

    LAN_IF_ADDR=`ip address show ${LAN_IF_NAME} | grep "\<inet\>" | head -n 1 | awk -F ' ' '{print $2}' | awk -F '/' '{print $1}'`
    if [ -z "${LAN_IF_ADDR}" ]; then
        echo "Can't find the lan ip!"
        return 1
    fi

    LAN_IF_MAC=`ip address show ${LAN_IF_NAME} | grep link/ether | awk -F ' ' '{print $2}'`
    if [ -z "${LAN_IF_MAC}" ]; then
        echo "Can't find the lan mac!"
        return 1
    fi
    
    NFT_CMD=`type -p nft`
    if [ -n "${NFT_CMD}" ]; then
        chain_check=$(nft list table inet fw4 2>/dev/null| grep -w 'chain input')
        if [ -n "${chain_check}" ]; then
            USE_NFT="true"
        fi
    fi
    
    return 0
}

check_rule()
{
    iptables -t mangle -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        echo "Mangle chain ${XUNYOU_CHAIN} hasn't been configurated!"
        return 1
    fi
    
    iptables -t mangle -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Mangle rule hasn't been configurated in chain PREROUTING"
        return 1
    fi
    
    iptables -t nat -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        echo "Nat chain ${XUNYOU_CHAIN} hasn't been configurated!"
        return 1
    fi
    
    iptables -t nat -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Nat rule hasn't been configurated in chain PREROUTING"
        return 1
    fi
    
    return 0
}

config_acc_rule()
{
    local src_ip=$1
    local set_udp_flag=$2
    
    if [ "${USE_NFT}" == "true" ]; then
        nft add table ip ${NFT_XUNYOU_TABLE} >/dev/null 2>&1
        nft add chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_DNAT_CHAIN} { type nat hook prerouting priority dstnat - 10  \; policy accept \; } >/dev/null 2>&1
        nft add chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_UDP_CHAIN} { type filter hook prerouting priority dstnat - 20 \; policy accept \; } >/dev/null 2>&1
        
        nft add table ip6 ${NFT_XUNYOU_TABLE6} >/dev/null 2>&1 
        nft add chain ip6 ${NFT_XUNYOU_TABLE6} ${NFT_XUNYOU_DROP_CHAIN} { type filter hook prerouting priority dstnat - 10  \; policy accept \; } >/dev/null 2>&1 
    else
        iptables -t nat -N ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t nat -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t mangle -N ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t mangle -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t mangle -N ${XUNYOU_TCP_DNS_CHAIN} >/dev/null 2>&1
        iptables -t mangle -I INPUT -i ${LAN_IF_NAME} -j ${XUNYOU_TCP_DNS_CHAIN} >/dev/null 2>&1
    fi
    
    ip rule add from ${src_ip} fwmark 0x199/0x199 table 199 >/dev/null 2>&1
    ip rule add from ${src_ip} table 198 >/dev/null 2>&1
    ip route add default via 172.19.199.1 dev tun199 table 199 >/dev/null 2>&1
    
    if [ "${USE_NFT}" == "true" ]; then
        nft add rule ip6 ${NFT_XUNYOU_TABLE6} ${NFT_XUNYOU_DROP_CHAIN} udp dport 53 drop >/dev/null 2>&1
        nft add rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_DNAT_CHAIN} ip saddr ${src_ip}  iifname ${LAN_IF_NAME} udp dport 53 dnat to 8.8.8.8:53 >/dev/null 2>&1
        nft add rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_UDP_CHAIN} ip saddr ${src_ip} iifname ${LAN_IF_NAME} udp dport 53 meta mark set 0x199 >/dev/null 2>&1

        if [ "${set_udp_flag}" = "1" ]; then
            nft insert rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_UDP_CHAIN} ip saddr ${src_ip} iifname ${LAN_IF_NAME} udp dport 1025-65535 meta mark set 0x199 >/dev/null 2>&1
            nft add rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_UDP_CHAIN} ip saddr ${src_ip} iifname ${LAN_IF_NAME} ip protocol icmp meta mark set 0x199 >/dev/null 2>&1
            #conntrack -D -s ${src_ip} >/dev/null 2>&1
        fi
        
        nft insert rule inet fw4 srcnat oifname "tun199" return >/dev/null 2>&1
        nft insert rule inet fw4 input iifname "tun199" accept >/dev/null 2>&1
        nft insert rule inet fw4 forward iifname "tun199" accept >/dev/null 2>&1
        nft insert rule inet fw4 forward oifname "tun199" accept >/dev/null 2>&1
    else
        ip6tables -t mangle -A PREROUTING -p udp --dport 53 -j DROP >/dev/null 2>&1
        iptables -t mangle -A ${XUNYOU_TCP_DNS_CHAIN} -s ${src_ip} -p tcp -m tcp --dport 53 -j DROP >/dev/null 2>&1
        iptables -t nat -A ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 53 -j DNAT --to-destination 8.8.8.8 >/dev/null 2>&1 
        iptables -t mangle -A ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 53 -j MARK --set-xmark 0x199/0x199 >/dev/null 2>&1 
        #主机
        if [ ${set_udp_flag} == "1" ]; then
            iptables -t mangle -I ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 1025:65535 -j MARK --set-xmark 0x199/0x199 >/dev/null 2>&1
            iptables -t mangle -A ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p icmp -j MARK --set-xmark 0x199/0x199 >/dev/null 2>&1
            #conntrack -D -s ${src_ip} >/dev/null 2>&1
        fi

        iptables -t nat -I POSTROUTING -o tun199 -j RETURN >/dev/null 2>&1
        iptables -t filter -I INPUT -i tun199 -j ACCEPT >/dev/null 2>&1
        iptables -t filter -I FORWARD -i tun199 -j ACCEPT >/dev/null 2>&1
        iptables -t filter -I FORWARD -o tun199 -j ACCEPT >/dev/null 2>&1
    fi
}

config_cdn_rule()
{
    local src_ip=$1
    local dst_ip=$2
    local dst_port=$3
    local redirect_port=$4

    if [ "${USE_NFT}" == "true" ]; then
        nft list chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            nft add chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        fi

        local num=`nft list chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_CDN_CHAIN} | grep $dst_port`
        if [ -z "${num}" ]; then
            nft add rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_CDN_CHAIN} tcp dport ${dst_port} redirect to ${redirect_port} >/dev/null 2>&1
        fi

        nft insert rule ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_DNAT_CHAIN} ip saddr ${src_ip} ip daddr ${dst_ip} tcp dport ${dst_port} jump ${NFT_XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    else
        iptables -t nat -n -L ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        if [ $? -ne 0 ];then
            iptables -t nat -N ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        fi

        iptables -t nat -C ${XUNYOU_CDN_CHAIN} -p tcp --dport ${dst_port} -j REDIRECT --to-ports ${redirect_port} >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            iptables -t nat -A ${XUNYOU_CDN_CHAIN} -p tcp --dport ${dst_port} -j REDIRECT --to-ports ${redirect_port} >/dev/null 2>&1
        fi

        iptables -t nat -I ${XUNYOU_CHAIN} -s ${src_ip} -d ${dst_ip} -p tcp --dport ${dst_port} -j ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    fi


}

clear_rule()
{
    local handel
    local handle_list
    ip route flush table 198 >/dev/null 2>&1
    ip rule delete table 198 >/dev/null 2>&1
    
    ip route flush table 199 >/dev/null 2>&1
    ip rule delete table 199 >/dev/null 2>&1
    
    if [ "${USE_NFT}" == "true" ]; then
        nft delete table ip ${NFT_XUNYOU_TABLE} >/dev/null 2>&1
        nft delete table ip6 ${NFT_XUNYOU_TABLE6} >/dev/null 2>&1
        
        handle_list=`nft -a -y list chain inet fw4 srcnat | grep -E "(tun199).*# handle" | awk '{print $(NF)}'`
        for handel in `echo ${handle_list}`
        do
            nft delete rule inet fw4 srcnat handle ${handel} >/dev/null 2>&1
        done
        
        handle_list=`nft -a -y list chain inet fw4 forward | grep -E "(tun199).*# handle" | awk '{print $(NF)}'`
        for handel in `echo ${handle_list}`
        do
            nft delete rule inet fw4 forward handle ${handel} >/dev/null 2>&1
        done
        
        handle_list=`nft -a -y list chain inet fw4 input | grep -E "(tun199).*# handle" | awk '{print $(NF)}'`
        for handel in `echo ${handle_list}`
        do
            nft delete rule inet fw4 input handle ${handel} >/dev/null 2>&1
        done
    else
        ip6tables -t mangle -D PREROUTING -p udp --dport 53 -j DROP >/dev/null 2>&1
        
        iptables -t mangle -F ${XUNYOU_TCP_DNS_CHAIN} >/dev/null 2>&1
        iptables -t mangle -D INPUT -i ${LAN_IF_NAME} -j ${XUNYOU_TCP_DNS_CHAIN} >/dev/null 2>&1
        iptables -t mangle -X ${XUNYOU_TCP_DNS_CHAIN} >/dev/null 2>&1

        iptables -t mangle -F ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t nat -F ${XUNYOU_CHAIN} >/dev/null 2>&1
        
        iptables -t nat -F ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        iptables -t nat -X ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
        
        iptables -t nat -D POSTROUTING -o tun199 -j RETURN >/dev/null 2>&1
        iptables -t filter -D INPUT -i tun199 -j ACCEPT >/dev/null 2>&1
        iptables -t filter -D FORWARD -o tun199 -j ACCEPT >/dev/null 2>&1
        iptables -t filter -D FORWARD -i tun199 -j ACCEPT >/dev/null 2>&1
        
        iptables -t mangle -D PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t mangle -X ${XUNYOU_CHAIN} >/dev/null 2>&1
        
        iptables -t nat -D PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t nat -X ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi

    rm -f ${ACC_CONF}
}

init_rule()
{
    if [ -d ${WORK_DIR} ]; then
        if [ "${USE_NFT}" == "true" ]; then
            nft add table ip ${NFT_XUNYOU_TABLE} >/dev/null 2>&1 
            nft add chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_DNAT_CHAIN} { type nat hook prerouting priority dstnat - 10  \; policy accept \; } >/dev/null 2>&1 
            nft add chain ip ${NFT_XUNYOU_TABLE} ${NFT_XUNYOU_UDP_CHAIN} { type filter hook prerouting priority dstnat - 20 \; policy accept \; } >/dev/null 2>&1 
            
            nft add table ip6 ${NFT_XUNYOU_TABLE6} >/dev/null 2>&1 
            nft add chain ip6 ${NFT_XUNYOU_TABLE6} ${NFT_XUNYOU_DROP_CHAIN} { type filter hook prerouting priority dstnat - 10  \; policy accept \; } >/dev/null 2>&1 
        else
            iptables -t mangle -N ${XUNYOU_CHAIN} >/dev/null 2>&1 
            iptables -t mangle -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
            iptables -t mangle -F ${XUNYOU_CHAIN} >/dev/null 2>&1
            
            iptables -t nat -N ${XUNYOU_CHAIN} >/dev/null 2>&1
            iptables -t nat -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
            iptables -t nat -F ${XUNYOU_CHAIN} >/dev/null 2>&1
        fi
    fi
    return 0
}

#called by fw3 framework
reload_fw() {
    #将ACC_CONF文件改名，防止清除规则的时候该文件被删除
    if [ -f ${ACC_CONF} ]; then
        mv ${ACC_CONF} ${ACC_CONF}.tmp
    fi

    clear_rule
    init_rule

    if [ -f ${ACC_CONF}.tmp ]; then
        mv ${ACC_CONF}.tmp ${ACC_CONF}

        while read line
        do
            $line
        done < ${ACC_CONF}
    fi
}

#called by plugin-app
close_fw(){
    clear_rule
}

#called by plugin-app
check_fw(){
    check_rule
}

env_init
if [ $? -ne 0 ]; then
    exit 1
fi

case $1 in
    "reload")
        reload_fw
        ;;
    "close")
        close_fw
        ;;
    "status")
        check_fw
        if [ $? -ne 0 ]; then
            #2 is 规则丢失，需要重启进程
            exit 2
        fi
        ;;
    "start_acc")
        close_fw
        config_acc_rule $2 $3
        ;;
    "stop_acc")
        close_fw
        ;;
    "config_cdn")
        config_cdn_rule $2 $3 $4 $5
        ;;
    *)
    #others
    ;;
esac

exit 0